@@ -8,31 +8,62 @@
 #
 package Jedi::Plugin::Template::Role;
 
-# ABSTRACT: Jedi Plugin for Template Toolkit (Role)
+# ABSTRACT: Role imported by Jedi::Plugin::Template
 
-use Moo::Role;
-our $VERSION = '0.04';    # VERSION
+use strict;
+use warnings;
+
+our $VERSION = '1.000';    # VERSION
 use Template;
 use Path::Class;
 use feature 'state';
 use MIME::Types qw/by_suffix/;
-use Carp qw/croak/;
+use Carp;
 use IO::Compress::Gzip qw(gzip);
 use HTTP::Date qw/time2str/;
 use Digest::SHA qw/sha1_base64/;
+use File::ShareDir ':ALL';
 
-before 'jedi_app' => sub {
-    my ($jedi) = @_;
+sub _jedi_template_check_path {
+    my ($path) = @_;
 
-    $jedi->get( qr{.*}x, $jedi->can('_jedi_dispatch_public_files') );
+    return if !defined $path;
+
+    return if ( !-d dir( $path, 'views' ) ) || ( !-d dir( $path, 'public' ) );
+
+    return dir($path)->absolute;
+}
+
+sub _jedi_template_setup_path {
+    my ($jedi_app) = @_;
+
+    my $class = ref $jedi_app;
+    my $dist  = $class;
+    $dist =~ s/::/-/gx;
+
+    my $template_dir =
+
+        # config dir
+        _jedi_template_check_path(
+        $jedi_app->jedi_config->{$class}{template_dir} )
+        //
+
+        # dist_dir
+        _jedi_template_check_path( eval { dist_dir($dist) } );
+
+    croak "No template dir found, please setup one !"
+        if !defined $template_dir;
+
+    $jedi_app->jedi_config->{$class}{template_dir} = dir($template_dir);
 
     return;
-};
+}
 
 sub _jedi_dispatch_public_files {
-    my ( $jedi, $request, $response ) = @_;
-    my $file
-        = file( $jedi->jedi_app_root, 'public', $request->env->{PATH_INFO} );
+    my ( $jedi_app, $request, $response ) = @_;
+    my $class = ref $jedi_app;
+    my $file  = file( $jedi_app->jedi_config->{$class}{template_dir},
+        'public', $request->env->{PATH_INFO} );
     return 1 if !-f $file;
 
     my ( $mime_type, $encoding ) = by_suffix($file);
@@ -62,24 +93,31 @@ sub _jedi_dispatch_public_files {
     return;
 }
 
-has '_jedi_template_views' => ( is => 'lazy' );
-
-sub _build__jedi_template_views {
-    my ($jedi) = @_;
-    return dir( $jedi->jedi_app_root, 'views' );
-}
+use Moo::Role;
 
 has 'jedi_template_default_layout' => ( is => 'rw' );
 
+before 'jedi_app' => sub {
+    my ($jedi_app) = @_;
+
+    _jedi_template_setup_path($jedi_app);
+
+    $jedi_app->get( qr{.*}x, \&_jedi_dispatch_public_files );
+
+    return;
+};
+
 sub jedi_template {
-    my ( $jedi, $file, $vars, $layout ) = @_;
-    $layout //= $jedi->jedi_template_default_layout;
+    my ( $jedi_app, $file, $vars, $layout ) = @_;
+    $layout //= $jedi_app->jedi_template_default_layout;
     $layout = 'none' if !defined $layout;
+    my $class = ref $jedi_app;
+    my $template_views
+        = dir( $jedi_app->jedi_config->{$class}{template_dir}, 'views' );
 
     my $layout_file;
     if ( $layout ne 'none' ) {
-        $layout_file
-            = file( $jedi->_jedi_template_views, 'layouts', $layout );
+        $layout_file = file( $template_views, 'layouts', $layout );
         if ( !-f $layout_file ) {
             $layout      = 'none';
             $layout_file = undef;
@@ -89,22 +127,22 @@ sub jedi_template {
     state $cache = {};
     if ( !exists $cache->{$layout} ) {
         my @tpl_options = (
-            INCLUDE_PATH => [ $jedi->_jedi_template_views ],
+            INCLUDE_PATH => [ $template_views->absolute->stringify ],
             ABSOLUTE     => 1,
         );
 
         if ( $layout ne 'none' ) {
-            push @tpl_options, WRAPPER => $layout_file->stringify;
+            push @tpl_options, WRAPPER => $layout_file->absolute->stringify;
         }
 
         $cache->{$layout} = Template->new(@tpl_options);
     }
 
     my $tpl_engine = $cache->{$layout};
-    my $view_file = file( $jedi->_jedi_template_views, $file );
+    my $view_file = file( $template_views, $file );
 
     my $ret = "";
-    $tpl_engine->process( $view_file->stringify, $vars, \$ret )
+    $tpl_engine->process( $view_file->absolute->stringify, $vars, \$ret )
         or croak $tpl_engine->error();
 
     return $ret;
@@ -118,39 +156,20 @@ __END__
 
 =head1 NAME
 
-Jedi::Plugin::Template::Role - Jedi Plugin for Template Toolkit (Role)
+Jedi::Plugin::Template::Role - Role imported by Jedi::Plugin::Template
 
 =head1 VERSION
 
-version 0.04
+version 1.000
 
-=head1 ATTRIBUTES
+=head1 DESCRIPTION
 
-=head2 jedi_template_default_layout
-
-if you want to set a default layout, use this attribute.
-
-	$jedi->jedi_template_default_layout('main.tt');
-
-=head1 METHODS
-
-=head2 jedi_template
-
-This method will use L<Template> to process your template.
-
-	$jedi->jedi_template($file, $vars);
-	$jedi->jedi_template($file, $vars, $layout);
-
-The layout use the jedi_template_default_layout by default.
-You can also remove any layout, using the value "none".
-
-The file is a file inside the subdir "views". The subdir "views" is located on the root of your apps, in
-the same directory than the "config.*".
+Check L<Jedi::Plugin::Template> for documentation
 
 =head1 BUGS
 
 Please report any bugs or feature requests on the bugtracker website
-https://tasks.celogeek.com/projects/perl-modules-jedi
+https://github.com/celogeek/perl-jedi-plugin-template/issues
 
 When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
